/*
HTML Tactics
Open source, cross-browser javascript tactics game engine written entirely by David Maletz.
Does not use any external libraries (maybe a few code snippets I had used before), written entirely from scratch.

This is probably the largest project I have written in javascript (as it is not my primary language), and may have some bugs - use at your own risk (GPL - see license.txt for the license).
I have removed all the bugs I could find, but there may be more, especially if you use functions in ways I did not expect, since there is no private or protected in javascript.
Also note that there are no threads in javascript, and no blocking commands - so a lot of functions use callbacks to let you know when they are done.
This can be annoying, and make code difficult to read, but it works once you get the hang of it.
Beware of problems with callbacks, and also functions that can only be called once at any given time (like moveToAnim).

There is little documentation in the code - if you have questions, feel free to email me at terra0nova@hotmail.com, and I may answer them.
But the best way to figure out how to use the code is to view the code in action.
Look at the code for the game I made with this engine - Shadow of Time 4 (sot4.js and all of the map files) which I have included in the sot4 folder.

Developing a game using HTML Tactics:

Make one file ({name}.js) with game specific defines. This includes items, weapons, classes, any functions you need, the intro screen, etc.
In the file you wish to include the game, add the following lines of code to the head:
<script type='text/javascript'>var GAME_NAME = "{name}";</script>
<script type='text/javascript' src='game_includes.js'></script>

And the following code in the body where you want the game window:
<script type='text/javascript'>loadGame();</script>

I also recommend you add <body onunload="stopBGM(); stopSE();"> to instantly stop the music when the page is unloaded, but the plugin will stop eventually even if you don't add it.

You make new games in javascript, but you also have the World Builder (builder.html) which can create maps in a gui and then save/load/send them.
Saving stores the map as plain text in a textarea, and loading loads plain text previously saved from that textarea (tileset or map).
This is a little annoying, but javascript cannot save to your local machine.

Send will store the current map in a compressed format to the server in the folder 'maps', as long as maploader.php exists.
It will overwrite any file with the same name sent automatically (but you should really keep these maps saved locally - see above).
If you have multiple games, you can move these map files to a new folder and then specify that folder by adding "MAP_DIR = {dir}" in the {name}.js file, making the maps folder empty to start work on a new project.

It has a fairly simple GUI, and you can add events there which have javascript attached (which run when the map is loaded).
You can reference the current event in this script by the variable "event", and you can attach the following functions to it:
mclick - called when mouse clicks event,
stepon - called when player steps on the event,
bump - called when the player bumps the event,
actionkey - called when the player hits enter next to the event,
onhit - called when the event gets hit in battle,
ondeath - called when the event gets killed in battle - if defined, the event will not be removed on death unless manually removed in the ondeath function (removeEvent, fadeAnim),
inrange - called whenever the event or player moves, and passes the current distance between the player and event to the function,
useitem - called when the player uses and item on top of or next to the event, and passes the used item,
startbattle - called when a battle starts,
endbattle - called when a battle ends.

You can also store variables which will be saved when the game is by using global.name (saved between maps) or event.vars.name (saved only on current map).
If you want to save an object, you must define a getReference function which returns a string that upon evaluation returns the object.

You should probably look through the various js files, as they all have useful (and uncommented) utility functions (makeDoor, makeTransport, showShop, etc).
A lot of these functions have intuitive naming, and seeing them used in sot4 will help you figure out their functionality.
If you have a specific question, you can send me an email.

recover.html is also useful - it allows you to recover maps that were sent to the server.
Just type the map name, and click Recover, and it will output plain text to a textarea which can be loaded into the builder.
This includes everything but blocking information - that is lost when you send the map to the server.
But, since most maps have the same tileset, you will probably be able to find the blocking information on some other map.

When releasing a game, you should delete maploader.php, as this allows people to mess with your game's maps and upload files.
For extra security, you might want to add password protection to maploader.php or only allow certain IPs to send maps to it - these are simple changes that will keep your map files safe if others can access maploader.php.

That's everything I'm going to cover - and everything you need, except good and detailed documentation of all functions (which I admit would be useful, but I don't have the time nor desire to do so).

Enjoy HTML Tactics and Shadow of Time 4.

NOTE: Requires Java JRE installed to play music and sound effects.
This is because Java was the best and most efficient way to get cross platform music working in game (embeds were very slow to change tracks on some browsers).
Even without Java, the game will run fine, there will just be no sound.

Images and sounds used for Shadow of Time 4 are from the internet and RPGMaker, and so are not open source, and so are not distributed with the source code.

*/

var selected = null;
var cur_player = 0;

var animating = false, end_game = false, request_wait = false, msg_show = false, game_paused = false, inBattle = false, singlePlayer = true;

var showSponsor = false, START_FUNC = false, allowBattles = true;

function gamePaused(){return animating | end_game | request_wait | msg_show | game_paused;}

function pauseGame(){game_paused = true;} function resumeGame(){game_paused = false;}

function error(text){
	if(showSponsor) sponsor_sprite.style.width = '64px'; else sponsor_sprite.style.width = '0px';
	sponsor_sprite.framems = 200;
	msg_div.style.visibility = 'visible';
	msg_div.className = 'blackout';
	msg_div.onclick = function(){};
	msg_align.style.verticalAlign = 'middle';
	msg_div.callback = false;
	menuOpen = false;
	msg_text.innerHTML = '<font color=red><b>'+text+'</b></font>';
	end_game = true;
	uibar.innerHTML = "<div class='nameTag' style='color:#666666'><a href='javascript:showTitle();'>Return to Title</a></div>";
	uibar.style.visibility = 'visible';
}

function showMsg(text){
	if(end_game) return;
	if(showSponsor) sponsor_sprite.style.width = '64px'; else sponsor_sprite.style.width = '0px';
	sponsor_sprite.framems = 200;
	msg_div.style.visibility = 'visible';
	msg_div.className = 'blackout';
	msg_div.onclick = function(){};
	msg_align.style.verticalAlign = 'middle';
	msg_div.callback = false;
	menuOpen = false;
	msg_text.innerHTML = text;
	msg_show = true;
}
function closeMsg(){if(end_game) return; menuOpen = false; msg_show = false; msg_div.style.visibility = 'hidden';}

function showDialog(text, vpos, callback){
	if(end_game) return;
	if(!vpos) vpos = 'bottom';
	sponsor_sprite.style.width = '0px';
	msg_div.style.visibility = 'visible';
	msg_div.className = 'blankout';
	msg_div.onclick = closeDialog;
	msg_align.style.verticalAlign = vpos;
	msg_div.callback = callback;
	menuOpen = false;
	msg_text.innerHTML = text;
	msg_show = true;
}

function showDialogArray(text_ar, vpos, callback, idx){
	if(end_game) return;
	if(!idx) idx = 0;
	showDialog(text_ar[idx], vpos, function(){
		idx++; if(idx >= text_ar.length){if(callback) callback();}
		else showDialogArray(text_ar, vpos, callback, idx);
	});
}

function closeDialog(){
	if(end_game | !msg_show) return;
	menuOpen = false; 
	msg_show = false; msg_div.style.visibility = 'hidden';
	if(msg_div.callback){var func = msg_div.callback; msg_div.callback = false; func();}
}

function showChoice(text, choices, callbacks, vpos){
	if(end_game) return;
	if(!vpos) vpos = 'bottom';
	sponsor_sprite.style.width = '0px';
	msg_div.style.visibility = 'visible';
	msg_div.className = 'blankout';
	msg_div.onclick = function(){};
	msg_align.style.verticalAlign = vpos;
	msg_div.callback = false;
	msg_div.callbacks = callbacks;
	if(text != "") text = "<b>"+text+"</b><hr>";
	for(var i=0; i<choices.length; i++){
		if(i != 0) text += "<br>";
		text += "<a href='javascript:selectChoice("+i+");'>"+choices[i]+"</a>";
	}
	menuOpen = false;
	msg_text.innerHTML = text;
	msg_show = true;
}

function selectChoice(i){
	if(end_game | !msg_show) return;
	menuOpen = false; 
	msg_show = false; msg_div.style.visibility = 'hidden';
	if(msg_div.callbacks[i]){var func = msg_div.callbacks[i]; msg_div.callbacks = false; func();}
}

function checkboxLabel(obj){obj.previousSibling.checked=!obj.previousSibling.checked;}

var menuScreen, menuSel, PARTY=1, OPTIONS=2, EQUIPMENT=3, SKILLS=4, SAVE=5;

function getMenuSelected(){return ((menuScreen == EQUIPMENT) | (menuScreen == SKILLS))?menuSel:-1;}

function showOptions(){
	if(!menuOpen) return;
	menuScreen = OPTIONS;
	document.getElementById('rightTitle').innerHTML = "Options";
	var text = "";
	text += "<input type='checkbox' onclick='muteMusic(this.checked);'"+((musicMuted)?" checked":"")+"><div style='display:inline;' onclick='checkboxLabel(this); muteMusic(this.previousSibling.checked);'>&nbsp;Mute Music</div><br>";
	text += "<input type='checkbox' onclick='muteSound(this.checked);'"+((soundMuted)?" checked":"")+"><div style='display:inline;' onclick='checkboxLabel(this); muteSound(this.previousSibling.checked);'>&nbsp;Mute Sound</div><br>";
	text += "<input type='checkbox' onclick='doAnimTiles = !doAnimTiles; saveOptions();'"+((doAnimTiles)?" checked":"")+"><div style='display:inline;' onclick='checkboxLabel(this); doAnimTiles = !doAnimTiles; saveOptions();'>&nbsp;Tile Animations (disable if slow)</div><br>";
	text += "Animation Speed: <select onchange='speed_mod = this.value; saveOptions();'>";
	text += "<option value='0.2'"+((speed_mod == 0.2)?" selected":"")+">Very Fast</option>";
	text += "<option value='0.5'"+((speed_mod == 0.5)?" selected":"")+">Fast</option>";
	text += "<option value='1'"+((speed_mod == 1)?" selected":"")+">Normal</option>";
	text += "<option value='2'"+((speed_mod == 2)?" selected":"")+">Slow</option>";
	text += "<option value='5'"+((speed_mod == 5)?" selected":"")+">Very Slow</option>";
	text += "</select>";
	document.getElementById('rightPanel').innerHTML = text;
}

function showItem(s, func, stats){
	if(!s) return "";
	return "<table cellpadding=0 cellspacing=0><tr><td>"+getIcon(s.icon,func)+"</td><td style='padding-left:5px; font-size:12px;'><b><u>"+s.name+"</b></u>:&nbsp;"+s.description+((stats)?" "+s.skillStr(stats):"")+"</font></td></tr></table>";
}

function showEquipment(i){
	menuScreen = EQUIPMENT;
	menuSel = i;
	document.getElementById('rightTitle').innerHTML = party[i].name+"'s Equipment";
	var text = "<table cellpadding=0 cellspacing=0 width=100%>"+getMemberStatus(i, "");
	var lvlExp = nextLevelEXP(party[i].level-1);
	text += "<tr><td colspan=2 align=center><table><tr><td style='font-size:14px'><b>Level:</b> "+party[i].level+"</td><td>"+getBar("Yellow", party[i].exp-lvlExp, nextLevelEXP(party[i].level)-lvlExp, lvlExp)+"</td>";
	text += "<td rowspan=2 align=right><table border=1 class='statTable' cellpadding=2 cellspacing=1>";
	text += "<tr><th>V</th><th>D</th><th>A</th><th>M</th><th>R</th><th>W</th></tr><tr>";
	for(var s=0; s<party[i].stats.length; s++) text += "<td>"+Math.floor(party[i].stats[s]+0.5)+"</td>";
	text += "</tr></table></td></tr><tr><td colspan=2 align=center style='font-size:12px;'><b>Range:</b> "+party[i].range+"&nbsp;&nbsp;&nbsp;&nbsp;<b>AC:</b> "+party[i].getAC()+"&nbsp;&nbsp;&nbsp;&nbsp;<b>ATK AP:</b> "+party[i].getAtkAP()+"<br><b>Damage:</b> "+party[i].getDamageStr()+"</td></tr>";
	text += "</table></td></tr><tr><td colspan=2 style='border-top:2px ridge #cccccc;' valign='top' height=95%><table border=1 width=100% cellpadding=0 cellspacing=0>";
	for(var e=0; e<NUM_EQUIP; e++)
		text += "<tr><td width=10% height=36 align=right style='padding-left:2px;'><b>"+equipNames[e]+":</b></td><td width=90% style='cursor:pointer;' onclick='unequip("+i+","+e+");'>"+showItem(party[i].get(e), "unequip("+i+","+e+")")+"</td></tr>";
	text += "</table></td></tr></table>";
	document.getElementById('rightPanel').innerHTML = text;
}

function showSkills(i){
	if(!menuOpen) showMenu();
	menuScreen = SKILLS;
	menuSel = i;
	document.getElementById('rightTitle').innerHTML = party[i].name+"'s Skills";
	var text = "<table cellpadding=0 cellspacing=0 width=350 height=48>"+getMemberStatus(i, "")+"</table>";
	text += "<div style='border-top:2px ridge #cccccc; width:350px; height:250px; overflow:auto;'><table width=100% border=1 cellpadding=0 cellspacing=0>";
	for(var c=0; c<party[i].skills.length; c++)
		text += "<tr><td height=33 style='cursor:pointer;' onclick='castSkill(party["+i+"].skills["+c+"]);'>"+showItem(party[i].skills[c], "castSkill(party["+i+"].skills["+c+"])", party[i].stats)+"</td></tr>";
	if(party[i].skills.length == 0) text += "<tr><td height=33><i>"+party[i].name+" has no skills.</i></td></tr>";
	text += "</table></div>";
	document.getElementById('rightPanel').innerHTML = text;
}

function showUnit(sprite, func, i){
	return "<div class='unitView' onclick='"+func+";' style=\""+((sprite.substring(0, 3) == "097")?"background-position:-9px -14px;width:48px;":((parseInt(sprite.substring(0, 3)) >= 151)?"height:32px; ":""))+"cursor:pointer; background-image:url('images/Characters/"+sprite+"');\"></div>";
}

function showDetails(member){
	var obj = (member.unit)?member.unit:member;
	return "<div style='float:left;'>"+getBar('Red', obj.health, obj.maxhealth)+getBar('Blue', obj.ap, obj.maxap)+"</div>";
}

function getMemberStatus(i, func){
	return "<tr><td width=5%>"+showUnit(party[i].img, func, i)+"</td><td width=95%><table width=100%><tr><td>"+showDetails(party[i])+"</td>"+
		"<td><a href='javascript:showEquipment("+i+");'>Equipment</a></td><td><a href='javascript:showSkills("+i+");'>Skills</a></td></tr></table></td></tr>";
}

function showPartyPanel(){
	if(!menuOpen) return;
	menuScreen = PARTY;
	document.getElementById('rightTitle').innerHTML = "Party";
	var text = "<center>";
	for(var i=0; i<party.length; i++)
		text += "<table border=1 width=90%><tr><td colspan=2><a href='javascript:showEquipment("+i+");' style='color:black;'><b>"+party[i].name+" (Lv. "+party[i].level+")</a></b></td></tr>"+
		getMemberStatus(i, "showEquipment("+i+")")+"</table>";
	text += "</center>";
	document.getElementById('rightPanel').innerHTML = text;
}

var partyselectcallback;

function selectPartyMember(i){
	if(partyselectcallback) partyselectcallback(i);
}

function showPartySelector(callback){
	if(!menuOpen) return;
	menuScreen = PARTY;
	document.getElementById('rightTitle').innerHTML = "Select a Party Member";
	partyselectcallback = callback;
	var text = "<center>";
	for(var i=0; i<party.length; i++)
		text += "<table border=1 width=90%><tr><td colspan=2><a href='javascript:selectPartyMember("+i+");' style='color:black;'><b>"+party[i].name+" (Lv. "+party[i].level+")</a></b></td></tr>"+
		getMemberStatus(i, "selectPartyMember("+i+")")+"</table>";
	text += "</center>";
	document.getElementById('rightPanel').innerHTML = text;
}

function saveGamePanel(){
	if(!menuOpen) return;
	menuScreen = SAVE;
	document.getElementById('rightTitle').innerHTML = "Save Game";
	if(!inBattle){
		var text = ""; var saves = getSavedGames();
		for(var i=0; i<saves.length; i++){
			if(i != 0) text += "<br>";
			text += "<a href=\"javascript:overwriteSave('"+saves[i]+"');\">"+saves[i]+"</a>";
		}
		if(saves.length != 0) text += "<br><br>";
		text += "<b>New Save:</b> <input type='text' id='newsavetext' size=20 onfocus='textareafocused=true;' onblur='textareafocused=false;'>&nbsp;&nbsp;<a href=\"javascript:saveGame(document.getElementById('newsavetext').value)\">(create)</a>";
		document.getElementById('rightPanel').innerHTML = text;
	} else document.getElementById('rightPanel').innerHTML = "<i>You can not save during a battle.</i>";
}

function refreshMenu(){
	document.getElementById('goldCount').innerHTML = getIcon(GOLD_ICON)+" "+inventory.gold;
	if(menuScreen == PARTY) showPartyPanel();
	else if(menuScreen == OPTIONS) showOptions();
	else if(menuScreen == EQUIPMENT) showEquipment(menuSel);
	else if(menuScreen == SKILLS) showSkills(menuSel);
	else if(menuScreen == SAVE) saveGamePanel();
}

var menuOpen = false, inDialog = false;
function showMenu(){
	if(menuOpen){menuScreen = PARTY; closeMsg(); return;}
	if(gamePaused() | allowMovePause) return;
	if(inBattle){
		if(cur_player != player) return;
		var tmp = selected;
		deselect();
		select(tmp);
	}
	sponsor_sprite.style.width = '0px';
	msg_div.style.visibility = 'visible';
	msg_div.className = 'blankout';
	msg_div.onclick = function(){if(!inDialog){menuScreen = PARTY; closeMsg();}};
	msg_align.style.verticalAlign = 'middle';
	msg_div.callback = false;
	var text = "<table border=1 width=98% height=90% onmouseover='inDialog = true;' onmouseout='inDialog = false;'><tr height=10%><td align=center><b>Game Menu</b></td>";
	text += "<td id='rightTitle' style='font-weight:bold;' align=center></td></tr><tr height=80%><td valign='top'>";
	text += "<a href='javascript:showPartyPanel();'>Party</a>";
	text += "<br><a href='javascript:showInventory();'>Inventory</a>";
	text += "<br><a href='javascript:showOptions();'>Options</a>";
	text += "<br><a href='javascript:saveGamePanel();'>Save Game</a>";
	text += "<br><a href='javascript:endGame();'>End Game</a>";
	text += "<br><a href='javascript:closeMsg();'>Return To Game</a>";
	text += "<br><br></td><td rowspan=2 align=center width=350><div id='rightPanel' style='text-align:center; width:350px; height:300px; overflow:auto;'>";
	text += "</div></td></tr><tr height=10%><td valign='bottom' style='border:0px; font-weight:bold' id='goldCount'>"+getIcon(GOLD_ICON)+" "+inventory.gold+"</td></tr></table>";
	msg_text.innerHTML = text;
	menuOpen = true;
	msg_show = true;
	showPartyPanel();
	showInventory();
}

function endGame(){
	if(confirm("Do you really want to end this game?")) showTitle();
}

var victoryMusic, battleMusic, sceneMusic;

function winBattle(novictory){
	if(!party[0].unit || party[0].unit.health <= 0){gameOverScreen(); return;}
	for(var i=0; i<party.length; i++)
		for(var c=0; c<party[i].stats.length; c++){party[i].stats[c] -= party[i].modifiers[c]; party[i].updateStats();}
	var units = playerUnits[player];
	for(var i=0; i<units.length; i++){
		var u = units[i];
		u.range = u.maxrange;
	}
	updatePartyMembers();
	deselect();
	selarea.style.visibility = 'hidden';
	aoearea.style.visibility = 'hidden';
	var endfunc = function(){
		inBattle = false;
		clearUIbar();
		if(sceneMusic) playBGM(sceneMusic);
		for(var i=0; i<eventList.length; i++){
			var e = eventList[i];
			if(e.endbattle) e.endbattle();
			if(e.player == player && !e.member && !e.mapEvent) e.fadeAnim();
		}
	}
	if(novictory) endfunc(); else{if(victoryMusic) playBGM(victoryMusic, 1); showDialog('<b>You Won A Battle!</b>', 'middle', endfunc);}
	if(showSponsor) sponsor_sprite.style.width = '64px'; else sponsor_sprite.style.width = '0px';
	sponsor_sprite.framems = 200;
	if(!novictory) hideParty();
}

setTimeout("rejuvenate()", 10000);

function rejuvenate(){
	if(!inBattle){
		var changed = false;
		for(var i=0; i<party.length; i++){
			var unit = ((party[i].unit)?party[i].unit:party[i]);
			if(unit.ap < unit.maxap){changed = true; unit.ap++;}
		}
		if(changed){
			if(!invShow) refreshUI();
			if(menuOpen){refreshMenu();}
		}
	}
	setTimeout("rejuvenate()", 10000);
}

var battleReady;

function testBattleReady(){
	battleReady++;
	if(battleReady == 2){
		inBattle = true;
		refreshUI();
		startTurn();
	}
}

var bossBattle = false;

function startBattle(){
	if(gamePaused() | !allowBattles) return;
	if(playerUnits[AI].length == 0) return;
	bossBattle = false;
	cur_player = Math.floor(Math.random()*(players.length-2)+0.5)+1;
	if(!BGMct) sceneMusic = BGM; else sceneMusic = false;
	if(battleMusic) playBGM(battleMusic);
	for(var i=0; i<eventList.length; i++){
		var e = eventList[i];
		if(e.startbattle) e.startbattle();
	}
	for(var i=0; i<party.length; i++) party[i].modifiers = zeroArray(party[i].stats.length);
	battleReady = 0;
	showDialog('<font color=red><b>Battle Start!</b></font>', 'middle', testBattleReady);
	if(showSponsor) sponsor_sprite.style.width = '64px'; else sponsor_sprite.style.width = '0px';
	sponsor_sprite.framems = 100;
	showParty(testBattleReady);
}

var htmlreqobj;
function loadReqObj(){
	try{
		if(window.XMLHttpRequest)
			htmlreqobj = new XMLHttpRequest();
		else if(window.ActiveXObject)
			htmlreqobj = new ActiveXObject("Microsoft.XMLHTTP");
		if(!htmlreqobj) error("Unable to connect to server.");
	}catch(e){error("Unable to connect to server.");}
}

function sendHandler(){
	if(htmlreqobj.readyState != 4) return;
	if(htmlreqobj.status==200){
		request_wait = false;
		closeMsg();
		var valid = false;
		var text = htmlreqobj.responseText;
		try{
			eval(text);
			if(!valid) error('Game out of sync with server.');
		} catch(e){error("Bad Data from Server");}
		
	} else error("Connection Failure, Server Status: <b>"+htmlreqobj.status+"</b>");
}

function sendReq(url){
	//Multiplayer incomplete.
	/*try{
		loadReqObj();
		htmlreqobj.onreadystatechange = sendHandler;
		htmlreqobj.open("GET", url, true);
		request_wait = true;
		htmlreqobj.send(null);
	}catch(e){error("Unable to connect to server.");}*/
}

function getX(obj){return obj.x;}
function getY(obj){return obj.y;}
function setX(obj, x){obj.x = x; obj.style.left = (x*TILE_SIZE-obj.width*0.5+16)+'px';}
function setY(obj, y){
	if(y == obj.y) return;
	obj.y = y;
	if(obj.parentNode) obj.parentNode.removeChild(obj);
	layers[obj.layer*worldY+Math.max(0, Math.ceil(obj.y)-((obj.layer == 2)?3:0))].appendChild(obj);
	obj.style.bottom = (Math.floor(y*TILE_SIZE+0.5))+'px';
}


var attackSel = false, attack_weapon, attack_apcost, attack_item, attacker_stats;
var selBorder = 3;

function attack(){
	if((selected == null) | !inBattle) return;
	var weapon = barehanded;
	if(selected.member && selected.member.get(WEAPON)) weapon = selected.member.get(WEAPON);
	else if(selected.weapon) weapon = selected.weapon;
	useWeapon(weapon);
};

function castScroll(scroll){
	castSkill(scroll.skill, "use", scroll.name, (inBattle)?useAP:0);
}

function castSkill(skill, useStr, name, apcost){
	if(!useStr){useStr = "cast"; attack_item = false;} if(!name) name = skill.name; if(!apcost) apcost = skill.apcost;
	var menuSel = getMenuSelected();
	if(!inBattle){
		if((menuSel == -1 || (!party[menuSel].unit && party[menuSel].health <= 0) || (party[menuSel].unit && party[menuSel].unit.health <= 0)) && !attack_item) return;
		if(skill.skilltype == HEAL_SKILL){
			var unit = false;;
			if(!attack_item){
				unit = (party[menuSel].unit)?party[menuSel].unit:party[menuSel];
				if(unit.ap < apcost){showUIStatus("Not Enough AP to "+useStr+" "+name+"."); return;}
			}
			if(!menuOpen) showMenu();
			showPartySelector(function(i){
				var heal = (party[i].unit)?party[i].unit:party[i];
				if(heal.health == 0){showUIStatus("Cannot heal unconscious party members."); return;}
				if(heal.health == heal.maxhealth){showUIStatus(heal.name+" is already at full health."); return;}
				if(!attack_item) unit.ap -= apcost;
				if(skill.animation) skill.animation();
				heal.health += skill.baseHeal((unit)?unit.stats:[5,5,5,5,5,5]);
				heal.health = Math.min(heal.health, heal.maxhealth);
				refreshUI();
				if(!attack_item) showSkills(menuSel);
				else {refreshMenu(); removeItem(attack_item);}
			});
		} else if(skill.skilltype == REVIVE_SKILL){
			var unit;
			if(!attack_item){
				unit = (party[menuSel].unit)?party[menuSel].unit:party[menuSel];
				if(unit.ap < apcost){showUIStatus("Not Enough AP to "+useStr+" "+name+"."); return;}
			}
			if(!menuOpen) showMenu();
			showPartySelector(function(i){
				var heal = (party[i].unit)?party[i].unit:party[i];
				if(heal.health != 0){showUIStatus("Cannot revive conscious party members."); return;}
				if(!attack_item) unit.ap -= apcost;
				if(skill.animation) skill.animation();
				heal.health = Math.max(1, Math.floor(heal.maxhealth*0.1));
				refreshUI();
				if(!attack_item) showSkills(menuSel);
				else {refreshMenu(); removeItem(attack_item);}
			});
		} else showUIStatus(name+" can only be used in battle.");
		return;
	}
	if(menuSel != -1 && party[menuSel].unit && party[menuSel].unit.health > 0){
		closeMsg();
		if(party[menuSel].unit.ap < apcost){showUIStatus("Not Enough AP to "+useStr+" "+name+"."); return;}
		deselect();
		select(party[menuSel].unit);
		useWeapon(skill, apcost);
	} else if(selected != null){
		if(selected.ap < apcost){showUIStatus("Not Enough AP to "+useStr+" "+name+"."); return;}
		useWeapon(skill, apcost);
	}
}

function playerTurn(){return inBattle & (cur_player == player);}

function useWeapon(weapon, apcost){
	if(!playerTurn() | gamePaused() | (selected == null)) return;
	if(!apcost){apcost = weapon.apcost; attack_item = false;}
	if(selected.ap < apcost){showUIStatus("Not Enough AP to Attack."); return;}
	attack_weapon = weapon;
	attack_apcost = apcost;
	var x = getX(selected), y = getY(selected);
	paths = new Array(1);
	paths[0] = new Array();
	weapon.getRange(paths[0], x, y);
	attackSel = true;
	showSelArea(x, y, weapon.maxrange, "atk");
	paths[0] = new Array();
	weapon.getAoE(paths[0], 0, 0);
	aoearea.innerHTML = getRangeHTML(0, 0, weapon.maxaoerange, "aoe");
	updateAnimElemList();
}

function getRangeHTML(x, y, range, classPrefix){
	var row = (2*range+1);
	var len = row*row;
	var grid = new Array(len);
	for(var i=0; i<len; i++) grid[i] = false;
	for(var r=0; r<paths.length; r++){
		var ar = paths[r];
		for(var c=0; c<ar.length; c++)
			grid[(ar[c][1]-y+range)*row+(ar[c][0]-x+range)] = true;
	}
	var text = "";
	for(var r=0; r<paths.length; r++){
		var ar = paths[r];
		for(var c=0; c<ar.length; c++){
			var idx = (ar[c][1]-y+range)*row+(ar[c][0]-x+range);
			var borders = "";
			var left=false, right=false, top=false, bottom=false;
			if(idx-1 < 0 || !grid[idx-1]){borders += "<div class='borderLeft "+classPrefix+"Border'></div>";left=true;}
			if(idx+1 >= len || !grid[idx+1]){borders += "<div class='borderRight "+classPrefix+"Border'></div>";right=true;}
			if(idx-row < 0 || !grid[idx-row]){borders += "<div class='borderBottom "+classPrefix+"Border'></div>";bottom=true;}
			if(idx+row >= len || !grid[idx+row]){borders += "<div class='borderTop "+classPrefix+"Border'></div>";top=true;}
			if(top & left) borders += "<div class='topLeft "+classPrefix+"Border'></div>";
			if(top & right) borders += "<div class='topRight "+classPrefix+"Border'></div>";
			if(bottom & left) borders += "<div class='bottomLeft "+classPrefix+"Border'></div>";
			if(bottom & right) borders += "<div class='bottomRight "+classPrefix+"Border'></div>";
			text += "<div class='"+classPrefix+"cell' moveUp='1' style='"+getMoveUpTile()+" bottom:"+(ar[c][1]*TILE_SIZE)+"px; left:"+(ar[c][0]*TILE_SIZE)+"px;'>"+borders+"</div>";
		}
	}
	return text;
}

function showSelArea(x, y, range, classPrefix){
	selarea.innerHTML = getRangeHTML(x, y, range, classPrefix);
	selarea.style.visibility = 'visible';
	updateAnimElemList();
}

function select(obj){
	if(!playerTurn() | gamePaused()) return;
	if(selected == obj){deselect(); return;}
	var x = getX(obj), y = getY(obj), range = obj.range;
	computePaths(x, y, range, obj.blocker);
	showSelArea(x, y, range, "move");
	selected = obj;
	barUnitShowing = null;
	showUI(selected)
}

function deselect(){
	if(!inBattle) return;
	selarea.style.visibility = 'hidden';
	aoearea.style.visibility = 'hidden';
	attackSel = false;
	selected = null;
	clearUIbar();
}

var movePath;
var moveStep, moveUnit, moveCallback;

function moveToAnim(unit, callback, cx, cy){
	if(unit.x != Math.floor(unit.x)) setX(unit, Math.floor(unit.x)+((getDirX(unit.facing) == 1)?1:0));
	if(unit.y != Math.floor(unit.y)) setY(unit, Math.floor(unit.y)+((getDirY(unit.facing) == 1)?1:0));
	var x = getX(unit), y = getY(unit);
	var m_dist = dist(x, y, cx, cy);
	if(m_dist == 0){if(callback) callback(); return;}
	movePath = new Array();
	Pathfind(x, y, cx, cy, unit.blocker, movePath);
	if(movePath.length > 0)
		moveAnimHelper(unit, callback, cx, cy);
	else {
		Pathfind(x, y, cx, cy, 0, movePath);
		moveAnimHelper(unit, callback, cx, cy);
	}
}

function doMoveAnim(unit, callback, cx, cy, r){
	if(unit.x != Math.floor(unit.x)) setX(unit, Math.floor(unit.x)+((getDirX(unit.facing) == 1)?1:0));
	if(unit.y != Math.floor(unit.y)) setY(unit, Math.floor(unit.y)+((getDirY(unit.facing) == 1)?1:0));
	movePath = new Array(r+1);
	movePath[r] = -1;
	retrievePath(cx, cy, r-1);
	moveAnimHelper(unit, callback, cx, cy);
}

function moveAnimHelper(unit, callback, cx, cy){
	if(unit.blocker == 1){
		worldGrid[getY(unit)*worldX+getX(unit)] = null;
		worldGrid[cy*worldX+cx] = unit;
	}
	startMove(unit, callback);
}

function startMove(unit, callback){
	if(unit) moveUnit = unit; else moveUnit = selected;
	moveCallback = callback;
	if(moveUnit == null) return;
	animating = true;
	moveStep = 0;
	setTimeout("moveAnim()", moveUnit.framems*speed_mod);
}

function getDirX(dir){return (dir == 1)?-1:(dir == 2)?1:0;}
function getDirY(dir){return (dir == 0)?-1:(dir == 3)?1:0;}

function moveAnim(){
	if(!moveUnit) return;
	var x = getX(moveUnit);
	var y = getY(moveUnit);
	var dir;
	if((x != Math.floor(x)) | (y != Math.floor(y))) dir = moveUnit.facing;
	else{
		dir = movePath[moveStep++];
		if(dir == -1){
			if(moveUnit.animfunc != 1) moveUnit.frame = 0;
			moveUnit.style.backgroundPosition = (-moveUnit.frame*moveUnit.width)+'px '+(-moveUnit.facing*moveUnit.height)+'px';
			animating = false;
			if(moveUnit == selected){
				var tmp = moveUnit;
				deselect();
				select(tmp);
			}
			if(moveCallback){var func = moveCallback; moveCallback = false; func();}
			if(request_wait) showMsg('Waiting for server...');
			return;
		}
		moveUnit.facing = dir;
	}
	var step_size = moveUnit.stepsize;
	x += step_size*getDirX(dir);
	y += step_size*getDirY(dir);
	setX(moveUnit, x);
	setY(moveUnit, y);
	var frame = moveUnit.frame;
	if(moveUnit.animfunc == 0) frame = (frame+1)%4;
	moveUnit.style.backgroundPosition = (-frame*moveUnit.width)+'px '+(-moveUnit.facing*moveUnit.height)+'px';
	moveUnit.frame = frame;
	setTimeout("moveAnim()", moveUnit.framems*speed_mod);
}

function dist(x1, y1, x2, y2){return Math.abs(x1-x2)+Math.abs(y1-y2);}
function distance(e1, e2){return dist(e1.x, e1.y, e2.x, e2.y);}

function retrievePath(x, y, r){
	if(r < 0) return;
	var ar = paths[r];
	for(var c=0; c<ar.length; c++){
		var cx = ar[c][0], cy = ar[c][1];
		if(dist(x, y, cx, cy) == 1){
			movePath[r] = (x > cx)?2:(x < cx)?1:(y > cy)?3:0;
			retrievePath(cx, cy, r-1);
			return;
		}
	}
}

function moveToCell(cx, cy){
	if(selected == null) return false;
	var x = getX(selected), y = getY(selected);
	var m_dist = dist(x, y, cx, cy);
	if(m_dist == 0) return false;
	var range = selected.range;
	for(var r=m_dist; r<paths.length; r++){
		var ar = paths[r];
		for(var c=0; c<ar.length; c++)
			if((ar[c][0] == cx) & (ar[c][1] == cy)){
				movePath = new Array(r+1);
				movePath[r] = -1;
				retrievePath(cx, cy, r-1);
				if(selected.blocker == 1){
					worldGrid[y*worldX+x] = null;
					worldGrid[cy*worldX+cx] = selected;
				}
				selected.range = range-r;
				if(!singlePlayer) sendReq('server.php?character='+0+'&move='+movePath+'&x='+cx+'&y='+cy);
				startMove();
				return true;
			}
	}
	return false;
}

//mouse handling

function getMouseCoords(e){
	var x, y;
	if(!e){
		x = window.event.offsetX;
		y = window.event.offsetY;
	} else {
		x = e.layerX;
		y = e.layerY;
	}
	return [x, 480-y];
}

function startTurn(){
	if(!inBattle | gamePaused()) return;
	if(players[cur_player] != NONE)
		showFloatMsg("<font style='font-size:48px; color:"+playerColors[cur_player]+";'>"+playerNames[cur_player]+"'s&nbsp;Turn</font>");
	pauseGame();
	setTimeout("doStartTurn()", 400);
}

function doStartTurn(){
	resumeGame();
	var p = (cur_player == 0)?players.length-1:cur_player-1;
	var units = playerUnits[p];
	for(var i=0; i<units.length; i++){
		var u = units[i];
		u.range = u.maxrange;
		u.ap = u.maxap;
	}
	if(players[cur_player] == LOCAL) return;
	if(cur_player == AI){if(menuOpen) closeMsg(); handleUnitAI(0);return;}
	cur_player = (cur_player+1)%players.length;
	clearUIbar();
	startTurn();
}

function endTurn(){
	if(!inBattle | gamePaused()) return;
	if(cur_player == player){
		cur_player = (cur_player+1)%players.length;
		deselect();
		startTurn();
	}
}

var barUnitShowing = null;

function battleTurnOp(){return "<a href='javascript:endTurn();'>End Turn</a>&nbsp;&nbsp;"+((bossBattle)?"":"<a href='javascript:escapeBattle();'>Escape</a>&nbsp;&nbsp;");}

function escapeBattle(){
	if(!inBattle || bossBattle || cur_player != player) return;
	if(Math.random() < 0.3){endTurn(); playSE("audio/sound/Evasion1.wav");}
	else{winBattle(true); playSE("audio/sound/Escape.wav");}
}

function showUIStatus(text){
	if(end_game) return;
	invShow = false;
	var text = "<div class='nameTag' style='color:#666666'>"+text+"</div><div class='rightText'>";
	if(inBattle & (cur_player == player)) text += battleTurnOp();
	text += "<a href='javascript:showInventory();'>Inventory</a>&nbsp;&nbsp;<a href='javascript:showMenu();'>Menu</a></div>";
	uibar.innerHTML = text;
	setTimeout("refreshUI()", 3000);
}

function clearUIbar(){
	if(end_game) return;
	invShow = false;
	barUnitShowing = null;
	var text = "";
	if(inBattle) text = "<div class='nameTag' style='color:#"+playerColors[cur_player]+"'>"+playerNames[cur_player]+"'s Turn</div>";
	text += "<div class='rightText'>";
	if(inBattle & (cur_player == player)) text += battleTurnOp();
	text += "<a href='javascript:showInventory();'>Inventory</a>&nbsp;&nbsp;<a href='javascript:showMenu();'>Menu</a></div>";
	uibar.innerHTML = text;
}

function getBar(bartype, val, max, base){
	var per = Math.floor(100*val/max);
	if(!per){per = 0;}
	if(base) val += base;
	return "<div class='bar'><div class='bar"+bartype+"' style='width:"+per+"%;'></div><div class='barnum'>"+val+"</div></div>";
}

function closeInventory(){invShow = false; refreshUI();}

function refreshUI(){
	var obj = barUnitShowing;
	barUnitShowing = null;
	if(obj == null){if(invShow) showInventory(); else clearUIbar();} else showUI(obj);
}

function showUI(obj){
	if(end_game | (barUnitShowing == obj)) return;
	if(obj == null || obj == blockerObj || obj.uname == ''){if(selected == null){clearUIbar(); return;} else obj = selected;}
	invShow = false;
	barUnitShowing = obj;
	var text = "<div class='nameTag' style='color:#"+playerColors[obj.player]+"'>"+obj.uname+((obj.level)?" (Lv. "+obj.level+")":"")+"</div>";
	if(obj.health > 0) text += "<div style='float:left; margin:5px 0 0 5px;'>"+getBar('Red', obj.health, obj.maxhealth)+getBar('Blue', obj.ap, obj.maxap)+"</div>";
	if(inBattle & (cur_player == player)){
		if(obj == selected) text += "<div class='leftText'><a href='javascript:attack();'>Attack</a>&nbsp;&nbsp;<a href='javascript:showSkillBar();'>Skills</a></div>";
		text += "<div class='rightText'>"+battleTurnOp();
	} else text += "<div class='rightText'>";
	text += "<a href='javascript:showInventory();'>Inventory</a>&nbsp;&nbsp;<a href='javascript:showMenu();'>Menu</a></div>";
	uibar.innerHTML = text;
}

function showFloatText(msg, x, y){
	var text = document.createElement("div");
	text.y = y*TILE_SIZE+16;
	text.style.left = (x*TILE_SIZE)+'px'; text.style.bottom = text.y+'px';
	text.className = 'floatMsg'; text.innerHTML = msg;
	text.stalpha = 15;
	text.style.filter="alpha(opacity=100)";
	gameWindow.appendChild(text);
	text.animfunc = 6;
	addAnimElem(text);
}

function showFloatMsg(msg){
	var text = document.createElement("div");
	text.y = (worldY/2-1)*TILE_SIZE+16;
	text.style.bottom = text.y+'px';
	text.style.left = '0px';
	text.style.width = '640px';
	text.className = 'floatMsg'; text.innerHTML = msg;
	text.stalpha = 15;
	text.style.filter="alpha(opacity=100)";
	gameWindow.appendChild(text);
	text.animfunc = 6;
	addAnimElem(text);
}

function damageEvent(event){
	if(event.health == 0) return;
	if(event.onhit){event.onhit();refreshUI();}
	var dmg = attack_weapon.baseDamage(attacker_stats, event.attribute, getX(event), getY(event), "#ff4036");
	if(event.member) for(var i=0; i<event.member.equipment.length; i++)
		if(event.member.equipment[i] && event.member.equipment[i].block) dmg = event.member.equipment[i].block(dmg, attack_weapon.attribute);
	var health = event.health-dmg;
	if(health <= 0){
		event.health = 0; if(event.ondeath){event.ondeath();refreshUI();} else{event.fadeAnim();}
		if(event.expgain && event.player == AI){
			var exp = event.expgain;
			for(var i=0; i<party.length; i++){
				if(!party[i].unit || party[i].unit.health <= 0) continue;
				party[i].exp += exp;
				var next = nextLevelEXP(party[i].level);
				var ct = 0;
				while(party[i].exp >= next){
					party[i].levelUp();
					ct++;
					next = nextLevelEXP(party[i].level);
				}
				if(ct > 0){
					lvlUpAnim(party[i].unit.x, party[i].unit.y);
					showFloatText("<font color='#5ab827'><b>LVL<br>UP!</b></font>", party[i].unit.x, party[i].unit.y);
				}
			}	
		}
	}
	else event.health = health;
}

function attackCell(x, y){
	var tmp = selected;
	deselect();
	select(tmp);
	if(!attack_weapon.inRange(x, y, getX(selected), getY(selected))) return;
	selected.faceCell(x, y);
	attacker_stats = (selected.member)?selected.member.stats:(selected.stats)?selected.stats:[5,5,5,5,5,5];
	if(attack_weapon.cast && !attack_weapon.cast(x,y)) return;
	if(attack_item) removeItem(attack_item);
	selected.ap -= attack_apcost;
	if(attack_weapon.animation) attack_weapon.animation(x, y);
	if(singlePlayer){
		attack_weapon.forEventsInAoE(x, y, (attack_weapon.handleevent)?attack_weapon.handleevent:damageEvent);
	} else {
		sendReq('server.php?character='+0+'&atk&x='+x+'&y='+y);
	}
	refreshUI();
	if(request_wait) showMsg('Waiting for server...');
}

var oldX=-1, oldY=-1;

function mouseMove(e){
	if(!cursor) return;
	var c = getMouseCoords(e);
	var x = Math.floor(c[0]*INV_TILE_SIZE), y = Math.floor(c[1]*INV_TILE_SIZE);
	if(x == oldX && y == oldY) return;
	oldX = x; oldY = y;
	cursor.style.bottom = Math.floor(c[1]*INV_TILE_SIZE)*TILE_SIZE+'px';
	cursor.style.left = Math.floor(c[0]*INV_TILE_SIZE)*TILE_SIZE+'px';
	cursor.style.visibility = 'visible';
	if(attackSel){
		if(attack_weapon.inRange(x, y, getX(selected),getY(selected))){
			aoearea.style.bottom = cursor.style.bottom;
			aoearea.style.left = cursor.style.left;
			aoearea.style.visibility = 'visible';
		} else aoearea.style.visibility = 'hidden';
	}
	var obj = getNamedEventByPos(x, y);
	if(obj == null) showUI(selected);
	else showUI(obj);
}

function getEventByName(name){
	name = name.toLowerCase();
	for(var i=0; i<eventList.length; i++)
		if(eventList[i].name.toLowerCase() == name) return eventList[i];
	return null;
}

function getNamedEventByPos(x, y){
	var max = null;
	for(var i=0; i<eventList.length; i++){
			var e = eventList[i];
			if(e.uname != '' && getX(e) == x && getY(e) == y && (max == null || (max.layer < e.layer && (max.player != player || e.player == player)) || e.player == player)) max = e;
		}
	return max;
}

function getEventByPos(x, y){
	var max = null;
	for(var i=0; i<eventList.length; i++){
		var e = eventList[i];
		if(getX(e) == x && getY(e) == y && (max == null || max.layer < e.layer)) max = e;
	}
	return max;
}

function forEventsByName(name, func){
	for(var i=0; i<eventList.length; i++)
		if(eventList[i].name == name) func(eventList[i]);
}

function forEventsByPos(x, y, func){
	for(var i=0; i<eventList.length; i++){
		var e = eventList[i];
		if(getX(e) == x && getY(e) == y) func(e);
	}
}

function clickEvent(x, y){
	var clicked = false;
	forEventsByPos(x, y, function(event){
		if(event && event.mclick){
			event.mclick();
			clicked = true;
		}
	});
	if(!clicked){
		var unit = getPlayerUnit();
		keyMovePath = new Array();
		keyMoveCt = 0; keyMoveX = x; keyMoveY = y; keyMoveTarget = getEventByPos(x, y);
		if((unit.x == Math.floor(unit.x)) & (unit.y == Math.floor(unit.y))) Pathfind(unit.x, unit.y, x, y, unit.blocker, keyMovePath); else keyComputePath = true;
		if(!moveAnimating){moveAnimating = true; keyMoveAnim();}
	}
	refreshUI();
}

function mouseClick(e){
	if(mapLoading || gamePaused() || !cursor) return;
	var c = getMouseCoords(e);
	var x = Math.floor(c[0]*INV_TILE_SIZE), y = Math.floor(c[1]*INV_TILE_SIZE);
	if(!inBattle){clickEvent(x, y); return;}
	var obj = getNamedEventByPos(x, y);
	if(attackSel){
		attackCell(x, y);
		return;
	}
	if((obj == null) | (obj == blockerObj)){
			if(!moveToCell(x, y))
				deselect();
	} else if(obj.player == player) select(obj);
}

//Arrow Keys when not in battle

/*
Events:
mclick, stepon, bump, actionkey, onhit, ondeath, inrange, useitem, startbattle, endbattle
*/

if(document.layers){document.captureEvents(Event.KEYDOWN); document.captureEvents(Event.KEYUP);}
document.onkeydown = doKeyDown;
document.onkeyup = doKeyUp;

var KEY_LEFT = 37, KEY_UP = 38, KEY_RIGHT = 39, KEY_DOWN = 40;

var keyDown = -1, moveAnimating = false, textareafocused = false, keyMovePath = new Array(), keyMoveCt, keyMoveX, keyMoveY, keyMoveTarget, keyComputePath = false;

function doKeyDown(e){
	if(mapLoading || allowMovePause || textareafocused || !cursor) return true;
	var nomove = gamePaused() | inBattle;
	var code = e?e.which:window.event.keyCode;
	if(code == 27){
		if(gameMenu.style.visibility == 'hidden') showMenu();
		return false;
	}
	if(code == 46){
		if(invShow && selitem) removeItem(getItem(parseInt(selitem.getAttribute('itemid'))));
		return false;
	}
	if((code == 13) | (code == 10)){
		if(msg_div.onclick) msg_div.onclick();
		if(!nomove){
			var obj = getPlayerUnit();
			var x = obj.x+getDirX(obj.facing);
			var y = obj.y+getDirY(obj.facing);
			for(var i=0; i<eventList.length; i++){
				var e = eventList[i];
				if(((e.x == x) & (e.y == y)) | ((e.x == obj.x) & (e.y == obj.y))) if(e.actionkey) e.actionkey();
			}
			refreshUI();
		}
	}
	if(nomove) return false;
	if((code >= KEY_LEFT) & (code <= KEY_DOWN)){keyDown = code; keyMovePath = new Array(); keyMoveCt = 0; if(!moveAnimating){moveAnimating = true; keyMoveAnim();}}
	return false;
}

function useItem(item){
	if(textareafocused) return true;
	if(gamePaused() && !menuOpen) return;
	if(item.use && !item.use()) return;
	if(!inBattle){
		var obj = getPlayerUnit();
		var x = obj.x+getDirX(obj.facing);
		var y = obj.y+getDirY(obj.facing);
		for(var i=0; i<eventList.length; i++){
			var e = eventList[i];
			if(((e.x == x) & (e.y == y)) | ((e.x == obj.x) & (e.y == obj.y))) if(e.useitem) e.useitem(item);
		}
	}
	refreshUI();
	if(menuOpen){refreshMenu();}
}

function doKeyUp(){
	keyDown = -1;
}

function keyMoveAnim(){
	var obj = getPlayerUnit();
	if(mapLoading || !obj){moveAnimating = false; if(obj) obj.setFrame(0); return;}
	var x = getX(obj);
	var y = getY(obj);
	var step_size = obj.stepsize;
	var dir, blocked = false, onpath = false;
	if((x != Math.floor(x)) | (y != Math.floor(y))) dir = obj.facing;
	else{
		if(gamePaused() | inBattle){moveAnimating = false; obj.setFrame(0); return;}
		if(keyComputePath | (keyMoveTarget && ((Math.floor(keyMoveTarget.x) != keyMoveX) | (Math.floor(keyMoveTarget.y) != keyMoveY)))){
			if(keyMoveTarget){keyMoveX = Math.floor(keyMoveTarget.x); keyMoveY = Math.floor(keyMoveTarget.y);}
			keyMovePath = new Array();
			Pathfind(obj.x, obj.y, keyMoveX, keyMoveY, obj.blocker, keyMovePath);
			keyMoveCt = 0;
			keyComputePath = false;
		}
		if(keyDown == KEY_LEFT) dir = 1;
		else if(keyDown == KEY_RIGHT) dir = 2;
		else if(keyDown == KEY_UP) dir = 3;
		else if(keyDown == KEY_DOWN) dir = 0;
		else if(keyMovePath.length != 0){
			onpath = true;
			if(keyMoveCt >= keyMovePath.length-1){
				if(keyMoveTarget && keyMoveTarget.actionkey) keyMoveTarget.actionkey();
				keyMovePath = new Array();
				obj.setFrame(0); moveAnimating = false; return;
			}
			else dir = keyMovePath[keyMoveCt++];
		}
		else {obj.setFrame(0); moveAnimating = false; return;}
		obj.setDir(dir);
		var tx = x+getDirX(dir);
		var ty = y+getDirY(dir);
		var bump;
		if(tx < 0 || tx >= worldX || ty < 0 || ty >= worldY) bump = null; else bump = worldGrid[ty*worldX+tx];
		if(bump != null) blocked = true;
		if((obj.blocker == 1) & (bump != null)){
			if(bump.bump){bump.bump(); refreshUI(); keyDown = -1;}
			dir = -1;
		} else {
			if((tx < 0) | (tx >= worldX)) dir = -1;
			else if((ty < 0) | (ty >= worldY)) dir = -1;
			else if(obj.blocker == 1){
				var idx = ty*worldX+tx;
				if(worldGrid[idx] != null) dir = -1; else {
					worldGrid[obj.y*worldX+obj.x] = null;
					worldGrid[idx] = obj;
				}
			}
		}
		if(!onpath){keyMoveX = -1; keyMoveY = -1; keyMoveTarget = null;}
	}
	x += step_size*getDirX(dir);
	y += step_size*getDirY(dir);
	setX(obj, x); setY(obj, y);
	var frame = obj.frame;
	if(obj.animfunc == 0) frame = (frame+1)%4;
	obj.style.backgroundPosition = (-frame*obj.width)+'px '+(-obj.facing*obj.height)+'px';
	obj.frame = frame;
	if(!blocked & (x == Math.floor(x)) & (y == Math.floor(y))){
		for(var i=0; i<eventList.length; i++){
			var e = eventList[i];
			var m_dist = dist(getX(e), getY(e), x, y);
			if((m_dist == 0) & (obj.blocker == 1) & (e.blocker != 1)) if(e.stepon) e.stepon();
			if(e.inrange) e.inrange(m_dist);
		}
		refreshUI();
	}
	if(blocked & (x == Math.floor(x)) & (y == Math.floor(y)) & onpath & keyMoveCt < keyMovePath.length-1){
		keyMovePath = new Array();
		Pathfind(obj.x, obj.y, keyMoveX, keyMoveY, obj.blocker, keyMovePath);
		keyMoveCt = 0;
	}
	setTimeout("keyMoveAnim()", obj.framems*speed_mod);
}

function startNewGame(){
	createCharacter(doGameStart);
}

function doGameStart(character){
	character.x = START_X; character.y = START_Y;
	inventory.gold = START_GOLD;
	addPartyMember(character);
	for(var i=0; i<START_ITEMS.length; i++)
		addItem(START_ITEMS[i]);
	if(START_FUNC) START_FUNC();
	else loadMap(START_MAP);
}

function closeTitle(){
	gameMenu.style.visibility = 'hidden';
	uibar.style.visibility = 'visible';
}

function showTitle(){
	while(creditsPane.hasChildNodes()) creditsPane.removeChild(creditsPane.firstChild);
	victoryMusic = VICTORY_MUSIC; battleMusic = BATTLE_MUSIC; sceneMusic = null;
	loseCallback = false;
	end_game = false;
	inBattle = false;
	closeMsg();
	clearUIbar();
	resetGlobal();
	playBGM(TITLE_MUSIC);
	displayTitle();
}

var loseCallback;

function gameOverScreen(){
	if(loseCallback) loseCallback();
	else {
		end_game = false;
		closeMsg();
		end_game = true;
		var blackout = document.createElement("div");
		blackout.className = 'blackout';
		blackout.style.backgroundImage = "url('"+DEATH_SCREEN+"')";
		playBGM(DEATH_BGM);
		creditsPane.appendChild(blackout);
		uibar.innerHTML = "<div class='nameTag' style='color:#666666'><a href='javascript:showTitle();'>Return to Title</a></div>";
		uibar.style.visibility = 'visible';
	}
}

function showTutorial(){if(!TUTORIAL_MAP) return; closeTitle(); loadMap(TUTORIAL_MAP);}

function displayTitle(){
	uibar.style.visibility = 'hidden';
	gameMenu.style.visibility = 'visible';
	msg_align.style.visibility = '';
	menuBox.style.width = '200px';
	menuBox.innerHTML = "<a href='javascript:startNewGame();'>Start New Game</a><br><a href='javascript:continueGame();'>Continue Game</a>"+((TUTORIAL_MAP)?"<br><a href='javascript:showTutorial();'>Show Tutorial</a>":"");
}

function makeDoor(door, keyid){
	if(!door.opensnd) door.opensnd = "audio/sound/Open1.wav";
	if(!door.closesnd) door.closesnd = "audio/sound/Close1.wav";
	if(!door.keysnd) door.keysnd = "audio/sound/Key.wav";
	if(keyid){door.keyid = keyid; door.vars.locked = true;}
	if(door.vars.open){door.setLayer(EVENT_P); door.setDir(3);} else door.setLayer(EVENT_B);
	door.actionkey = function(){
		if(this.vars.locked){showDialog("The Door is Locked.", "middle"); return;}
		this.callback = function(){
			if(this.facing == 3){this.vars.open = true; this.setLayer(EVENT_P);}
			else if(this.facing == 0){this.vars.open = undefined; this.setLayer(EVENT_B);}
		};
		if(this.facing == 0){if(this.opensnd) playSE(this.opensnd); this.rotateOnce();}
		else if(this.facing == 3) if(worldGrid[this.y*worldX+this.x] == null){if(this.closesnd) playSE(this.closesnd);  this.rotateOnce(REVERSE);}
	}
	if(keyid){
		door.useitem = function(item){
			if(!item.keyid || !this.keyid || this.facing != 0) return;
			if(item.keyid == this.keyid){
				this.vars.locked = !this.vars.locked;
				if(this.keysnd) playSE(this.keysnd);
				if(this.vars.locked) showDialog("The door has been locked.", "middle"); else showDialog("The door has been unlocked.", "middle");
			} else showDialog("The key does not work.", "middle");
		}
	}
}

function makeChest(chest, keyid){
	if(!chest.opensnd) chest.opensnd = "audio/sound/Switch2.wav";
	if(!chest.closesnd) chest.closesnd = "audio/sound/Switch1.wav";
	if(!chest.keysnd) chest.keysnd = "audio/sound/Key.wav";
	if(keyid){chest.keyid = keyid; chest.vars.locked = true;}
	chest.actionkey = function(){
		if(this.vars.locked){showDialog("The Chest is Locked.", "middle"); return;}
		if(this.facing == 0){if(this.opensnd) playSE(this.opensnd); this.callback = this.onopen; this.rotateOnce();}
		else if(this.facing == 3){if(this.closesnd) playSE(this.closesnd); this.callback = this.onclose; this.rotateOnce(REVERSE);}
	}
	if(keyid){
		chest.useitem = function(item){
			if(!item.keyid || !this.keyid || this.facing != 0) return;
			if(item.keyid == this.keyid){
				this.vars.locked = !this.vars.locked;
				if(this.keysnd) playSE(this.keysnd);
				if(this.vars.locked) showDialog("The chest has been locked.", "middle"); else showDialog("The chest has been unlocked.", "middle");
			} else showDialog("The key does not work.", "middle");
		}
	}
}

function makeTransport(event, map, x, y){
	if(typeof(x) == 'undefined')
		if(event.x == 0) x = worldX-1;
		else if(event.x == worldX-1) x = 0;
	if(typeof(y) == 'undefined')
		if(event.y == 0) y = worldY-1;
		else if(event.y == worldY-1) y = 0;
	event.stepon = function(){
		var unit = getPlayerUnit();
		if((event.x != keyMoveX | event.y != keyMoveY) & (((event.x == 0) & (unit.facing != 1)) | ((event.x == worldX-1) & (unit.facing != 2)) | ((event.y == 0) & (unit.facing != 0)) | ((event.y == worldY-1) & (unit.facing != 3)))) return;
		if(this.sound) playSE(this.sound); loadMap(map, x, y);
	}
}

function makeMonster(monster, aggro_range, expgain){
	if(!monster.aggrosnd) monster.aggrosnd = "audio/sound/Bell.wav";
	monster.aggro_range = aggro_range; monster.setPlayer(getPlayer(2));
	monster.expgain = expgain;
	monster.inrange = function(r){if(r <= this.aggro_range){if(this.aggrosnd) playSE(this.aggrosnd); startBattle();}}
}

function monsterReward(monster, chest_type, reward){
	if(!monster.deathsnd) monster.deathsnd = "audio/sound/Monster1.wav";
	monster.ondeath = function(){
		this.callback = function(){
			var chest = spawnBasicEvent('Chest', '17'+(4+monster.facing)+'-Chest0'+(1+monster.facing)+'.png', this.x, this.y);
			chest.setFrame(chest_type);
			makeChest(chest);
			chest.onopen = function(){if(reward) reward(); this.fadeAnim();}
		}
		if(this.deathsnd) playSE(this.deathsnd);
		this.fadeAnim();
	}
}

function showInn(cost){
	if(inBattle) return;
	var doshop = function(){
		showMsg(""); msg_align.style.visibility = 'hidden';
		if(!BGMct) sceneMusic = BGM; else sceneMusic = false;
		playBGM("audio/music/Inn2.mid", 1);
		setTimeout("closeInn()", 7000);
	}
	if(cost){
		if(cost > inventory.gold){showDialog("Not Enough Gold to Sleep Here.", 'middle'); return;}
		else showChoice("Would you like to sleep here for the night? It costs "+cost+" gold.", ["Yes", "No"], [doshop, function(){}], 'middle');
		inventory.gold -= cost;
	} else doshop();
}

function healParty(){
	for(var i=0; i<party.length; i++){
		party[i].health = party[i].maxhealth;
		party[i].ap = party[i].maxap;
		if(party[i].unit) party[i].unit.health = party[i].unit.maxhealth;
		if(party[i].unit) party[i].unit.ap = party[i].unit.maxap;
	}
}

function closeInn(){
	healParty();
	closeMsg();
	showDialog("You wake up in the morning feeling refreshed.", 'middle');
	refreshUI();
	if(sceneMusic) playBGM(sceneMusic);
	msg_align.style.visibility = '';
}

function invDir(d){if(d == 0) return 3; if(d == 3) return 0; if(d == 1) return 2; if(d == 2) return 1;}